
public class MerObj  {
	
	private String mer;
	private int[] fgrnd;
	private int[] offset;
	private int fSize;
	private float wmwScore;
	private float averageBindingScore;
	private String rc;	
	private int rcFlag;
	private int alignOffset;
	
	public MerObj (String kMer) {
		int expectedOccurence = Application.PROBES_BOUND * (Application.PROBE_SIZE - Application.MER_SIZE + 1) / (int)Math.pow(4, Application.MER_SIZE); 
		this.mer = kMer;
		this.fgrnd = new int[4*expectedOccurence];  // upper bound on the number of probes where each k-mer appears
		this.offset = new int[4*expectedOccurence]; // same
		this.fSize = 0;
		this.wmwScore = (float) -2;
		this.averageBindingScore = 0;
		this.rc = "";
		this.rcFlag = 0;
		this.alignOffset = MSAMotif.INVALID_ALIGNMENT;
	}
	
	/** Add instance of k-mer within probe #rank & its offset
	 * @param rank
	 * @param offset
	 */
	public void addInstance(int rank, int offset, float bindingScore) {
		this.fgrnd[this.fSize] = rank;
		this.offset[this.fSize] = offset;
		this.averageBindingScore += bindingScore;
		this.fSize++;
	}
	
	/** WMW score calculator
	 * @pre this.frgnd[].length > 0 (not empty)
	 * 
	 * @param probesNum = num of probes within input file.
	 */
	public void computeWMWAndAvg (int probesNum) {
		int f = (int) Math.ceil((float) this.fSize/2);
		int b = (int) Math.ceil((float) (probesNum-this.fSize)/2);
		int fIter = f;
		int bIter = b;
		int rhoF = 0;
		int rhoB = 0;
		int i = 0;
		int n = 0;
		/* if frgnd[0] > 1 then interval (1+2+...+(fgrnd[0]-1)) needs to be added to rhoB */
		if (this.fgrnd[i] > 1) {
			n = Math.min(bIter, this.fgrnd[i]-1);
			/* sum of sequence: a1+a2+...+an = n*(2*a1+d*(n-1))/2 */
			rhoB += (n)*(2+n-1)/2;
			bIter -= n;
		}
		/* iterating over this.fgrnd */
		while (fIter>0 || bIter>0) {
			/* adding fgrnd[i] to rhoF */
			if (fIter>0) {
				rhoF += this.fgrnd[i];
				fIter--;
			}
			/* adding sequence (fgrnd[i]+1 + fgrnd[i]+2 + ...) to rhoB */
			if (bIter>0) {
				// n = sequence size - how further after fgrnd[i]+1 we need to keep adding?
				if (this.fgrnd[i+1] == 0) { n = bIter; }
				else
					if (this.fgrnd[i] == this.fgrnd[i+1]) { n = 0; }
					else { n = Math.min(bIter, this.fgrnd[i+1]-this.fgrnd[i]-1); }
				
				/* sum of sequence: a1+a2+...+an = n*(2*a1+d*(n-1))/2 */
				rhoB += n*(2*(fgrnd[i]+1)+n-1)/2;
				bIter -= n;
			}
			if(this.fgrnd[i] > 0 && this.fgrnd[i+1] == 0) { break; } // we finished going over all ranks, and the rest of the array is populated with default zeroes 
			i++;
		}
		float score = (float) 1/(b+f);
		score *= (rhoB/b - rhoF/f);
		this.wmwScore = score;
		this.averageBindingScore = (float)(this.averageBindingScore / (float)this.fSize);
	}
	
	public Float getBerger() {
		return this.wmwScore;
	}
	
	public float getAvg() {
		return this.averageBindingScore;
	}
	
	public int getForegroundRank(int i) {
		return this.fgrnd[i];
	}
	
	public String getStr() {
		return this.mer.toString();
	}
	
	public int getForegroundSize() {
		return this.fSize;
	}
	
	public void setRc(String rc) {	
		this.rc = rc;
	}
	
	public void setRcFlag() {		
		this.rcFlag = 1;
	}
	
	public String getRcStr() {		
		return this.rc.toString();
	}
	
	public int getRcFlag() {		
		return this.rcFlag;
	}
	
	public int getProbeOffset(int i) {
		return this.offset[i];
	}
	
	public void setAlignOffset(int i) {
		this.alignOffset = i;
	}
	
	public int getAlignOffset() {
		return this.alignOffset;
	}
}